import * as tslib_1 from "tslib";
import * as React from 'react';
import { CommandButton, DefaultButton } from 'office-ui-fabric-react/lib/Button';
import { BaseComponent, css, format } from 'office-ui-fabric-react/lib/Utilities';
import { FILTER_CHANGE_EVENT, FILTER_APPLIED_EVENT } from '../../Utilities/Filter';
import * as Utils_Accessibility from '../../Utilities/Accessibility';
import * as VSSUIResources from '../../VSS.Resources.VSSUI';
import "./FilterBar.css";
var FilterBar = /** @class */ (function (_super) {
    tslib_1.__extends(FilterBar, _super);
    function FilterBar(props) {
        var _this = _super.call(this, props) || this;
        _this._onResize = function () {
            if (!_this._resizeTimeout) {
                _this._resizeTimeout = setTimeout(function () {
                    _this._resizeTimeout = null;
                    if (_this._isMounted) {
                        _this.setState({ filtersToShowStopIndex: FilterBar.RENDER_EVERYTHING });
                    }
                }, 100);
            }
        };
        _this._onPageLeft = function () {
            _this._hasMadeVisibleFilterAnnouncement = false;
            var startIndex = _this._startingFilterIndices.pop() || 0;
            _this._hasPagedLeft = true;
            _this.setState({ filtersToShowStartIndex: startIndex, filtersToShowStopIndex: FilterBar.RENDER_EVERYTHING });
        };
        _this._onPageRight = function () {
            _this._hasMadeVisibleFilterAnnouncement = false;
            _this._startingFilterIndices.push(_this.state.filtersToShowStartIndex);
            _this._hasPagedRight = true;
            _this.setState({ filtersToShowStartIndex: _this.state.filtersToShowStopIndex, filtersToShowStopIndex: FilterBar.RENDER_EVERYTHING });
        };
        _this._calculateFiltersToShowStopIndex = function () {
            var totalWidth = _this._rightElement.clientWidth;
            for (var i = 0; i < _this._childrenContainerElements.length; i++) {
                var elem = _this._childrenContainerElements[i];
                var elemWidth = elem.clientWidth + parseFloat(window.getComputedStyle(elem).marginRight);
                totalWidth += elemWidth;
                if (totalWidth > _this._filterBarElement.clientWidth) {
                    return _this.state.filtersToShowStartIndex + (i || 1); // make sure we show at least one filter.
                }
            }
            return _this.state.filtersToShowStartIndex + _this._childrenContainerElements.length;
        };
        _this._getAppliedFilterCount = function () {
            var count = 0;
            var filterState = _this.props.filter.getState();
            Object.keys(filterState).forEach(function (key) {
                if (filterState[key].value && (!Array.isArray(filterState[key].value) || filterState[key].value.length > 0)) {
                    count++;
                }
            });
            return count;
        };
        _this._onFilterChanged = function (changedState) {
            _this.setState({
                hasChangesToApply: _this.props.filter.hasChangesToApply(),
                hasChangesToReset: _this.props.filter.hasChangesToReset()
            });
        };
        _this._onFilterApplied = function (changedState) {
            _this.setState({
                hasChangesToApply: _this.props.filter.hasChangesToApply()
            });
        };
        _this._onReset = function () {
            _this.props.filter.reset();
            _this.focus();
        };
        _this._onApplyChanges = function () {
            _this.props.filter.applyChanges();
            _this.focus();
        };
        if (!props.filter) {
            throw new Error('Cannot create a FilterBar without a filter prop.');
        }
        _this._startingFilterIndices = [];
        _this._hasMadeVisibleFilterAnnouncement = false;
        _this._isMounted = false;
        _this.state = {
            hasChangesToReset: props.filter.hasChangesToReset(),
            hasChangesToApply: props.filter.hasChangesToApply(),
            filtersToShowStartIndex: 0,
            filtersToShowStopIndex: FilterBar.RENDER_EVERYTHING
        };
        return _this;
    }
    FilterBar.prototype.focus = function () {
        if (this._filterItemRefs && this._filterItemRefs.length > 0) {
            this._filterItemRefs[0].focus();
        }
    };
    FilterBar.prototype.forceUpdate = function () {
        _super.prototype.forceUpdate.call(this);
        if (this._filterItemRefs) {
            this._filterItemRefs.forEach(function (filterItem) { return filterItem.forceUpdate(); });
        }
    };
    FilterBar.prototype.componentDidMount = function () {
        this._events.on(this.props.filter, FILTER_CHANGE_EVENT, this._onFilterChanged);
        this._events.on(this.props.filter, FILTER_APPLIED_EVENT, this._onFilterApplied);
        window.addEventListener('resize', this._onResize);
        this.setState({ filtersToShowStopIndex: this._calculateFiltersToShowStopIndex() });
        this._isMounted = true;
        if (this.props.onMounted) {
            this.props.onMounted(this);
        }
    };
    FilterBar.prototype.componentWillReceiveProps = function (nextProps) {
        this.setState({
            hasChangesToApply: nextProps.filter.hasChangesToApply(),
            hasChangesToReset: nextProps.filter.hasChangesToReset()
        });
    };
    FilterBar.prototype.componentWillUnmount = function () {
        this._events.off(this.props.filter, FILTER_CHANGE_EVENT, this._onFilterChanged);
        this._events.off(this.props.filter, FILTER_APPLIED_EVENT, this._onFilterApplied);
        window.removeEventListener('resize', this._onResize);
        this._isMounted = false;
    };
    FilterBar.prototype.componentDidUpdate = function () {
        if (this.props.onRenderComplete) {
            this.props.onRenderComplete();
        }
        if (this._hasPagedLeft && this.state.filtersToShowStopIndex > 0) {
            if (this.state.filtersToShowStartIndex == 0) {
                this._nextButtonElem.focus();
            }
            this._hasPagedLeft = false;
        }
        if (this.state.filtersToShowStopIndex < 0) {
            var filtersToShowStopIndex = this._calculateFiltersToShowStopIndex();
            if (this._hasPagedRight &&
                filtersToShowStopIndex == this.state.filtersToShowStartIndex + this._childrenContainerElements.length) {
                this._prevButtonElem.focus();
            }
            this.setState({ filtersToShowStopIndex: this._calculateFiltersToShowStopIndex() });
            this._hasPagedRight = false;
        }
        else if (!this._hasMadeVisibleFilterAnnouncement) {
            Utils_Accessibility.announce(format(VSSUIResources.AnnonuceVisibleFilters, this.state.filtersToShowStartIndex + 1, this.state.filtersToShowStopIndex), true);
            this._hasMadeVisibleFilterAnnouncement = true;
        }
    };
    FilterBar.prototype.render = function () {
        var _this = this;
        var _a = this.props, children = _a.children, filter = _a.filter, className = _a.className;
        var _b = this.state, hasChangesToApply = _b.hasChangesToApply, hasChangesToReset = _b.hasChangesToReset, filtersToShowStopIndex = _b.filtersToShowStopIndex, filtersToShowStartIndex = _b.filtersToShowStartIndex;
        this._filterItemRefs = [];
        this._childrenContainerElements = [];
        var childrenWithProps = React.Children.map(children, function (child) {
            if (child === null) {
                return null;
            }
            var childElement = child;
            var childWithProps = React.cloneElement(childElement, {
                filter: childElement.props.filter || filter,
                ref: (function (elem) {
                    if (elem) {
                        _this._filterItemRefs.push(elem);
                    }
                })
            });
            return React.createElement("div", { className: 'vss-FilterBar--item', ref: function (elem) {
                    if (elem) {
                        _this._childrenContainerElements.push(elem);
                    }
                } }, childWithProps);
        });
        var canPageRight = filtersToShowStopIndex < childrenWithProps.length;
        var canPageLeft = filtersToShowStartIndex > 0;
        if (canPageRight || canPageLeft) {
            var endIndex = filtersToShowStopIndex > 0 ? filtersToShowStopIndex : childrenWithProps.length;
            childrenWithProps = childrenWithProps.slice(filtersToShowStartIndex, endIndex);
        }
        var resetText = filter.statesAreEqual(filter.getDefaultState(), {}) ?
            VSSUIResources.ClearFilterBarLinkText :
            VSSUIResources.ResetFilterBarLinkText;
        var filterCount = this._getAppliedFilterCount();
        return React.createElement("div", { className: css('vss-FilterBar', className) },
            React.createElement("div", { className: 'vss-FilterBar--list', ref: function (elem) { _this._filterBarElement = elem; } },
                childrenWithProps,
                React.createElement("div", { className: css('vss-FilterBar--right-items', { 'vss-FilterBar--right-items-align-right': (canPageLeft || canPageRight) }), ref: function (elem) { _this._rightElement = elem; } },
                    (canPageLeft || canPageRight) &&
                        React.createElement("div", { className: 'vss-FilterBar--page-button-container' },
                            React.createElement(DefaultButton, { className: "vss-FilterBar-page-button", componentRef: function (elem) { _this._prevButtonElem = elem; }, onClick: this._onPageLeft, disabled: !canPageLeft, ariaLabel: VSSUIResources.FilterPageLeftAriaLabel, iconProps: { iconName: "ChevronLeft" } }),
                            React.createElement(DefaultButton, { className: "vss-FilterBar-page-button", componentRef: function (elem) { _this._nextButtonElem = elem; }, onClick: this._onPageRight, disabled: !canPageRight, ariaLabel: VSSUIResources.FilterPageRightAriaLabel, iconProps: { iconName: "ChevronRight" } })),
                    React.createElement("div", { className: 'vss-FilterBar--action vss-FilterBar--action-clear' },
                        React.createElement(CommandButton, { disabled: !hasChangesToReset, onClick: this._onReset, iconProps: { iconName: 'Clear' }, ariaLabel: VSSUIResources.ClearFilterBarLinkAriaLabel }, resetText),
                        (filterCount > 0 || filtersToShowStopIndex == FilterBar.RENDER_EVERYTHING) &&
                            React.createElement("div", { className: 'vss-FilterBar--filter-count' }, filterCount)),
                    filter.usesApplyMode() &&
                        React.createElement("div", { className: 'vss-FilterBar--action vss-FilterBar--action-apply' },
                            React.createElement(CommandButton, { disabled: !hasChangesToApply, onClick: this._onApplyChanges, iconProps: { iconName: 'CheckMark' } }, VSSUIResources.ApplyChangesFilterBarText)))));
    };
    FilterBar.RENDER_EVERYTHING = -1;
    return FilterBar;
}(BaseComponent));
export { FilterBar };
