import * as tslib_1 from "tslib";
import * as React from 'react';
import * as VSSUIResources from '../../VSS.Resources.VSSUI';
import { TextField } from 'office-ui-fabric-react/lib/TextField';
import { FilterBarItem } from '../../Components/FilterBar/FilterBarItem';
var DEFAULT_MAX_TEXT_LENGTH = 200;
var DEFAULT_THROTTLE_WAIT = 200;
var TextFilterBarItem = /** @class */ (function (_super) {
    tslib_1.__extends(TextFilterBarItem, _super);
    function TextFilterBarItem() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._onClearInputKeyPress = function (ev) {
            if (ev.which === 13 /* enter */ || ev.which === 32 /* space */) {
                _this._resetText();
            }
        };
        _this._onClearInputClick = function () {
            _this._resetText();
        };
        _this._onTextChanged = function (text) {
            _this.setFilterValue({ value: text });
        };
        _this._onKeyDown = function (ev) {
            switch (ev.which) {
                case 13 /* enter */:
                    _this.props.filter.setFilterItemState(_this.props.filterItemKey, { value: _this.state.value });
                    _this.props.filter.applyChanges();
                    break;
                case 27 /* escape */:
                    _this.setFilterValue({ value: '' });
                    _this.setState({
                        value: ''
                    });
                    _this.props.filter.applyChanges();
                    break;
                default:
                    return;
            }
            // We only get here if the keypress has been handled.
            ev.preventDefault();
            ev.stopPropagation();
        };
        return _this;
    }
    TextFilterBarItem.prototype.focus = function () {
        return this._textField.focus();
    };
    TextFilterBarItem.prototype.render = function () {
        var _this = this;
        var value = this.state.value;
        var _a = this.props, placeholder = _a.placeholder, maxTextLength = _a.maxTextLength, onGetErrorMessage = _a.onGetErrorMessage;
        var iconProps = value ? {
            iconName: 'Clear',
            className: 'clear-filter-input',
            tabIndex: 0,
            onClick: this._onClearInputClick,
            onKeyPress: this._onClearInputKeyPress,
            role: "button",
            ariaLabel: VSSUIResources.FilterTextItemClearLabel
        } : undefined;
        return React.createElement(TextField, tslib_1.__assign({ componentRef: function (element) { return _this._textField = element; }, className: 'vss-FilterBar--item-text', value: value || '', placeholder: placeholder, ariaLabel: placeholder, onChanged: this._onTextChanged, onKeyDown: this._onKeyDown, maxLength: maxTextLength || DEFAULT_MAX_TEXT_LENGTH, iconProps: iconProps, onGetErrorMessage: onGetErrorMessage }, this.getExtraTextFieldProps()));
    };
    TextFilterBarItem.prototype.getExtraTextFieldProps = function () {
        return null;
    };
    TextFilterBarItem.prototype.getThrottleWait = function () {
        var throttleWait = this.props.throttleWait;
        return throttleWait === undefined ? DEFAULT_THROTTLE_WAIT : throttleWait;
    };
    TextFilterBarItem.prototype._resetText = function () {
        this.setFilterValue({ value: '' });
        this.focus();
    };
    return TextFilterBarItem;
}(FilterBarItem));
export { TextFilterBarItem };
