import * as tslib_1 from "tslib";
import * as React from "react";
import { BaseComponent, css } from "office-ui-fabric-react/lib/Utilities";
import { HubBreadcrumb } from "../../Components/HubHeader/HubBreadcrumb";
import { HubTitle } from "../../Components/HubHeader/HubTitle";
import { ItemIndicatorSet } from "../../ItemIndicator";
import { ObservableLike } from '../../Utilities/Observable';
import "./HubHeader.css";
var MAX_BREADCRUMB_WIDTH = '340px';
/**
 * Component that generates a standard VSTS page header.
 */
var HubHeader = /** @class */ (function (_super) {
    tslib_1.__extends(HubHeader, _super);
    function HubHeader(props) {
        var _this = _super.call(this, props) || this;
        _this.onHeaderItemClick = function () {
            if (_this._hubBreadcrumb && _this._hubBreadcrumb.pickList) {
                _this._hubBreadcrumb.pickList.toggleDropdown();
            }
        };
        _this.onSelectionChanged = function (selection) {
            var headerItemPicker = _this.state.headerItemPicker;
            var preventDefault = false;
            if (headerItemPicker && headerItemPicker.onSelectedItemChanged) {
                preventDefault = headerItemPicker.onSelectedItemChanged(selection.selectedItems && selection.selectedItems.length > 0 ? selection.selectedItems[0] : null) === true;
                if (_this._hubBreadcrumb && _this._hubBreadcrumb.pickList) {
                    _this._hubBreadcrumb.pickList.closeDropdown();
                }
            }
            return preventDefault;
        };
        _this.onHeaderPickerChanged = function (headerItemPicker) {
            _this.setState({ headerItemPicker: headerItemPicker });
        };
        _this.state = {
            headerItemPicker: ObservableLike.getValue(_this.props.headerItemPicker)
        };
        return _this;
    }
    HubHeader.prototype.componentWillMount = function () {
        ObservableLike.subscribe(this.props.headerItemPicker, this.onHeaderPickerChanged);
    };
    HubHeader.prototype.componentWillUnmount = function () {
        ObservableLike.unsubscribe(this.props.headerItemPicker, this.onHeaderPickerChanged);
    };
    HubHeader.prototype.componentWillReceiveProps = function (nextProps) {
        if (this.props.headerItemPicker !== nextProps.headerItemPicker) {
            this.setState({ headerItemPicker: ObservableLike.getValue(nextProps.headerItemPicker) });
            ObservableLike.unsubscribe(this.props.headerItemPicker, this.onHeaderPickerChanged);
            ObservableLike.subscribe(nextProps.headerItemPicker, this.onHeaderPickerChanged);
        }
    };
    /**
     * Gets the title (Breadcrumb control or HubTitle control, depending on whether or not breadcrumbitems were passed).
     */
    HubHeader.prototype.getTitle = function () {
        if ((this.props.breadcrumbItems && this.props.breadcrumbItems.length > 0) || this.props.headerItemPicker) {
            return this.getBreadcrumb();
        }
        else {
            if (this.props.title) {
                return (React.createElement(HubTitle, { titleText: this.props.title, iconProps: this.props.iconProps }));
            }
            else {
                console.log("Must use title or breadcrumb items + headerItemPicker");
                return null;
            }
        }
    };
    HubHeader.prototype.getBreadcrumb = function () {
        var _this = this;
        var breadcrumbItems = (this.props.breadcrumbItems) ? this.props.breadcrumbItems.slice() : [];
        var _a = this.props, maxBreadcrumbItemWidth = _a.maxBreadcrumbItemWidth, collapseBreadcrumb = _a.collapseBreadcrumb, nonInteractive = _a.nonInteractive, iconProps = _a.iconProps, pickListClassName = _a.pickListClassName, pickListMinWidth = _a.pickListMinWidth, title = _a.title, hubBreadcrumbAriaLabel = _a.hubBreadcrumbAriaLabel;
        var headerItemPicker = this.state.headerItemPicker;
        if (headerItemPicker && headerItemPicker.getListItem) {
            var selectedItem = headerItemPicker.selectedItem && headerItemPicker.getListItem(headerItemPicker.selectedItem);
            var selectedItemTitleText = (headerItemPicker.selectedItem && headerItemPicker.getTitleTextForItem) ?
                headerItemPicker.getTitleTextForItem(headerItemPicker.selectedItem) :
                selectedItem && selectedItem.name;
            var onClick = nonInteractive ? undefined : this.onHeaderItemClick;
            var titleItem = {
                key: 'title-item',
                text: title ? title : selectedItemTitleText,
                leftIconProps: iconProps,
                onClick: onClick,
                onSelectionChanged: this.onSelectionChanged,
                headerItemPicker: headerItemPicker,
                ariaLabel: headerItemPicker.ariaLabel
            };
            breadcrumbItems.push(titleItem);
        }
        else if (title) {
            var titleItem = {
                key: 'title-item',
                text: title,
                leftIconProps: iconProps,
            };
            breadcrumbItems.push(titleItem);
        }
        if (collapseBreadcrumb) {
            breadcrumbItems = breadcrumbItems.slice(breadcrumbItems.length - 1);
        }
        return React.createElement(HubBreadcrumb, { ref: function (hubBreadcrumb) { _this._hubBreadcrumb = hubBreadcrumb; }, items: breadcrumbItems, maxItemWidth: typeof maxBreadcrumbItemWidth === 'string' ? maxBreadcrumbItemWidth : MAX_BREADCRUMB_WIDTH, pickListClassName: pickListClassName, pickListMinWidth: pickListMinWidth, ariaLabel: hubBreadcrumbAriaLabel });
    };
    HubHeader.prototype.getIndicators = function () {
        var headerItemPicker = this.state.headerItemPicker;
        if (headerItemPicker && headerItemPicker.selectedItem && headerItemPicker.selectedItemIndicators && headerItemPicker.selectedItemIndicators.length) {
            return React.createElement(ItemIndicatorSet, { indicators: headerItemPicker.selectedItemIndicators.map(function (indicator) { return indicator.getItemIndicator(headerItemPicker.selectedItem); }) });
        }
        else {
            return null;
        }
    };
    HubHeader.prototype.render = function () {
        return (React.createElement("div", { className: css("vss-HubHeader") },
            this.getTitle(),
            this.getIndicators(),
            this.props.children));
    };
    return HubHeader;
}(BaseComponent));
export { HubHeader };
