import * as tslib_1 from "tslib";
import * as React from 'react';
import { format } from 'office-ui-fabric-react/lib/Utilities';
import { SelectionMode } from 'office-ui-fabric-react/lib/Selection';
import { PickListDropdown } from '../../Components/PickList/PickListDropdown';
import { FilterBarItem } from '../../FilterBar';
import * as VSSUIResources from '../../VSS.Resources.VSSUI';
var PickListFilterBarItem = /** @class */ (function (_super) {
    tslib_1.__extends(PickListFilterBarItem, _super);
    function PickListFilterBarItem() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._onClearClicked = function () {
            _this.setFilterValue({ value: [], operator: _this.state.operator });
            _this.focus();
        };
        _this._onSelectionChanged = function (selection) {
            _this.setFilterValue({ value: selection.selectedItems, operator: selection.selectedOperatorKey });
        };
        _this._getActions = function (items) {
            var actions = [];
            if (!_this.props.hideClearButton) {
                if (items && items.length > 0) {
                    _this._clearAction.disabled = !_this.state.value || _this.state.value.length === 0;
                    actions.push(_this._clearAction);
                }
            }
            return actions;
        };
        return _this;
    }
    PickListFilterBarItem.prototype.focus = function () {
        this._pickListDropdown.focus();
    };
    PickListFilterBarItem.prototype.forceUpdate = function () {
        if (this._pickListDropdown) {
            this._pickListDropdown.forceUpdate();
        }
    };
    PickListFilterBarItem.prototype.componentDidMount = function () {
        _super.prototype.componentDidMount.call(this);
        this._buildActions();
    };
    PickListFilterBarItem.prototype.render = function () {
        var _this = this;
        var placeholder = this.props.placeholder;
        var value = this.state.value;
        var formattedAriaLabel = null;
        var operators = this.props.operators;
        if (this.props.showOrAndOperators) {
            operators = [
                { name: VSSUIResources.FilterOperatorOr, key: "or" /* or */ },
                { name: VSSUIResources.FilterOperatorAnd, key: "and" /* and */ }
            ];
        }
        if (placeholder) {
            if (value && value.length > 0) {
                formattedAriaLabel = format(VSSUIResources.FilterDropdownLabel, placeholder) + ' {0}';
            }
            else {
                formattedAriaLabel = format(VSSUIResources.FilterDropdownLabelNoSelection, placeholder);
            }
        }
        return React.createElement(PickListDropdown, tslib_1.__assign({ className: 'vss-PickListDropdown--filterBar', pickListClassName: 'vss-PickList--filterBar', componentRef: function (pickListDropdown) { return _this._pickListDropdown = pickListDropdown; }, showSelectAll: this.props.selectionMode === SelectionMode.multiple, ariaLabelFormat: formattedAriaLabel }, this.props, { operators: operators, selectedItems: value, selectedOperatorKey: this.state.operator, getActions: this._getActions, onSelectionChanged: this._onSelectionChanged }));
    };
    PickListFilterBarItem.prototype._buildActions = function () {
        this._clearAction = {
            name: VSSUIResources.PickListClearActionText,
            iconProps: {
                iconName: "Clear",
                iconType: 0 /* fabric */
            },
            onClick: this._onClearClicked,
            ariaLabel: VSSUIResources.PickListClearActionAriaLabel
        };
    };
    return PickListFilterBarItem;
}(FilterBarItem));
export { PickListFilterBarItem };
