import * as tslib_1 from "tslib";
import * as React from "react";
import { VssDetailsList } from "../../VssDetailsList";
import { BaseComponent } from "office-ui-fabric-react/lib/Utilities";
import { Icon } from "office-ui-fabric-react/lib/Icon";
import { DetailsRow } from "office-ui-fabric-react/lib/DetailsList";
var TreeList = /** @class */ (function (_super) {
    tslib_1.__extends(TreeList, _super);
    function TreeList() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onRenderRow = function (props, renderer) {
            var treeListItem = props.item;
            if (treeListItem.isExpandable) {
                return React.createElement(ExpandCollapseRow, { expanded: treeListItem.isExpanded, treeListItem: treeListItem, rowProps: props, collapse: _this.onExpand, expand: _this.onCollapse });
            }
            return renderer(props);
        };
        _this.onExpand = function (treeListItem) {
            if (treeListItem.isExpanded) {
                _this.props.onItemToggled(treeListItem);
                return true;
            }
            return false;
        };
        _this.onCollapse = function (treeListItem) {
            if (!treeListItem.isExpanded) {
                _this.props.onItemToggled(treeListItem);
                return true;
            }
            return false;
        };
        return _this;
    }
    TreeList.prototype.render = function () {
        var _this = this;
        var _a = this.props, treeListItems = _a.treeListItems, primaryTreeColumnKey = _a.primaryTreeColumnKey, actionsColumnKey = _a.actionsColumnKey, getMenuItemProviders = _a.getMenuItemProviders, getMenuItems = _a.getMenuItems, allocateSpaceForActionsButtonWhileHidden = _a.allocateSpaceForActionsButtonWhileHidden;
        var columns = this.props.columns.slice();
        var primaryColumnIndex = 0;
        if (primaryTreeColumnKey !== undefined) {
            for (var i = 0; i < columns.length; i++) {
                if (columns[i].key === primaryTreeColumnKey) {
                    primaryColumnIndex = i;
                    break;
                }
            }
        }
        var initialColumnRender = columns[primaryColumnIndex].onRender;
        var newColumnRender = function (item, index, col) {
            return React.createElement(TreeListPrimaryCell, { onItemToggled: _this.props.onItemToggled, treeListItem: item, index: index, column: col, onColumnRender: initialColumnRender, indentPaddingInPixels: 30 });
        };
        columns[primaryColumnIndex] = tslib_1.__assign({}, columns[primaryColumnIndex], { onRender: newColumnRender });
        return React.createElement(VssDetailsList, { items: treeListItems, columns: columns, actionsColumnKey: actionsColumnKey, getMenuItems: getMenuItems, allocateSpaceForActionsButtonWhileHidden: allocateSpaceForActionsButtonWhileHidden, getMenuItemProviders: getMenuItemProviders, onRenderRow: this.onRenderRow, setKey: "0" });
    };
    return TreeList;
}(BaseComponent));
export { TreeList };
var TreeListPrimaryCell = /** @class */ (function (_super) {
    tslib_1.__extends(TreeListPrimaryCell, _super);
    function TreeListPrimaryCell() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    TreeListPrimaryCell.prototype.render = function () {
        var _this = this;
        var treeListItem = this.props.treeListItem;
        var indentPixels = this.props.indentPaddingInPixels * treeListItem.indentLevel;
        var collapseButton = treeListItem.isExpandable ? React.createElement(Icon, { iconName: treeListItem.isExpanded ? "ChevronDownSmall" : "ChevronRightSmall" }) : undefined;
        return React.createElement("div", { className: "tree-column-wrapper", onClick: function () { _this.props.onItemToggled(treeListItem); }, style: {
                "paddingLeft": indentPixels + "px"
            } },
            collapseButton,
            this.props.onColumnRender(this.props.treeListItem, this.props.index, this.props.column));
    };
    return TreeListPrimaryCell;
}(React.Component));
var ExpandCollapseRow = /** @class */ (function (_super) {
    tslib_1.__extends(ExpandCollapseRow, _super);
    function ExpandCollapseRow() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onKeyDown = function (event) {
            if (event.which === 39 /* KeyCode.RightArrow */) {
                if (_this.props.expand(_this.props.treeListItem)) {
                    event.preventDefault();
                    event.stopPropagation();
                }
            }
            else if (event.which === 37 /* KeyCode.LeftArrow */) {
                if (_this.rowComponent !== null && _this.detailsElement !== null && document.activeElement != _this.detailsElement) {
                    _this.rowComponent.focus();
                }
                else if (_this.props.collapse(_this.props.treeListItem)) {
                    event.preventDefault();
                    event.stopPropagation();
                }
            }
        };
        return _this;
    }
    ExpandCollapseRow.prototype.render = function () {
        var _this = this;
        return (React.createElement("div", { onKeyDown: this.onKeyDown, ref: function (rowElement) { _this.rowElement = rowElement; } },
            React.createElement(DetailsRow, tslib_1.__assign({}, this.props.rowProps, { "aria-expanded": this.props.expanded, ref: function (rowComponent) { _this.rowComponent = rowComponent; } }))));
    };
    ExpandCollapseRow.prototype.componentDidMount = function () {
        this.detailsElement = null;
        if (this.rowElement !== null) {
            this.detailsElement = this.rowElement.querySelector("[tabIndex='0']") || this.rowElement.children[0];
        }
    };
    return ExpandCollapseRow;
}(React.Component));
