import * as tslib_1 from "tslib";
import * as React from "react";
import { DefaultButton, PrimaryButton } from "office-ui-fabric-react/lib/Button";
import { BaseComponent } from "office-ui-fabric-react/lib/Utilities";
import "./ZeroData.css";
/**
 * Component for displaying helpful information when there is no data to show.
 */
var ZeroData = /** @class */ (function (_super) {
    tslib_1.__extends(ZeroData, _super);
    function ZeroData() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ZeroData.prototype.render = function () {
        return React.createElement(ZeroDataMultiple, { items: [this.props] });
    };
    return ZeroData;
}(BaseComponent));
export { ZeroData };
/**
 * Component for displaying helpful information when there is no data to show. This one displays
 * multiple (or one) ZeroDataItems.
 *
 * THIS CLASS IS NOT EXPORTED presently because no design for multiple ZeroDataItems has yet been
 * approved.
 */
var ZeroDataMultiple = /** @class */ (function (_super) {
    tslib_1.__extends(ZeroDataMultiple, _super);
    function ZeroDataMultiple() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ZeroDataMultiple.prototype.render = function () {
        var multiple = this.props.items.length > 1;
        return (React.createElement("div", { className: "vss-ZeroData " + (multiple ? "multiple" : "single") }, this.props.items.map(function (item, index) { return React.createElement(ZeroDataItem, { item: item, key: index, multiple: multiple }); })));
    };
    return ZeroDataMultiple;
}(BaseComponent));
/**
 * Represents a single item for the ZeroData component.
 */
var ZeroDataItem = /** @class */ (function (_super) {
    tslib_1.__extends(ZeroDataItem, _super);
    function ZeroDataItem() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ZeroDataItem.prototype.render = function () {
        var item = this.props.item;
        var secondary;
        if (typeof item.secondaryText === "string") {
            secondary = React.createElement("span", null, item.secondaryText);
        }
        else {
            secondary = item.secondaryText;
        }
        return (React.createElement("div", { className: "vss-ZeroDataItem" },
            React.createElement("img", { className: "vss-ZeroDataItem--image", src: item.imagePath, alt: item.imageAltText }),
            React.createElement("div", { className: "vss-ZeroDataItem--primary " + (this.props.multiple ? "ms-font-xl" : "ms-font-xxl") }, item.primaryText),
            secondary && React.createElement("div", { className: "vss-ZeroDataItem--secondary ms-font-m" }, secondary),
            this._renderAction(item)));
    };
    ZeroDataItem.prototype._renderAction = function (item) {
        if (!item.actionText) {
            return null;
        }
        var onActionClick = function (ev) {
            if (item.onActionClick) {
                item.onActionClick.call(null, ev, item);
            }
        };
        var buttonProps = {
            className: "vss-ZeroDataItem--action",
            text: item.actionText,
            onClick: onActionClick,
        };
        if (item.actionType === 0 /* ctaButton */) {
            return React.createElement(PrimaryButton, tslib_1.__assign({}, buttonProps));
        }
        else {
            return React.createElement(DefaultButton, tslib_1.__assign({}, buttonProps));
        }
    };
    return ZeroDataItem;
}(BaseComponent));
