import * as tslib_1 from "tslib";
import { ObservableArray } from "../Observable";
/**
 * Given an IItemProvider, implements an ObservableArray that contains the items provided by the provider.
 */
var ContributedItemArray = /** @class */ (function (_super) {
    tslib_1.__extends(ContributedItemArray, _super);
    function ContributedItemArray(itemProvider, mapDelegate) {
        var _this = _super.call(this, []) || this;
        // Let users attach an event listener before we call loadItems()
        setTimeout(function () {
            itemProvider.loadItems(function (items) {
                // When contributed items become available, update observable array
                _this.items = (items || []).map(mapDelegate);
            });
        }, 0);
        return _this;
    }
    return ContributedItemArray;
}(ObservableArray));
export { ContributedItemArray };
/**
 * Implements an IItemProvider that provides modified versions of items it gets from the source provider.
 */
var MutatedItemProvider = /** @class */ (function () {
    /**
     * @param provider provider that provides the source items
     * @param mutator function that modifies the items before providing them to this object's listener
     */
    function MutatedItemProvider(provider, mutator) {
        this.provider = provider;
        this.mutator = mutator;
    }
    MutatedItemProvider.prototype.loadItems = function (itemsUpdated) {
        var _this = this;
        this.provider.loadItems(function (items) { return itemsUpdated((items || []).map(_this.mutator)); });
    };
    return MutatedItemProvider;
}());
export { MutatedItemProvider };
